#include "Viewer.h"

//Linux
#include <QX11Info>
#include <GL/glx.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Xlib.h>
#include <Xw_Window.hxx>
#include <QColormap>

//occ
#include <Aspect_DisplayConnection.hxx>
#include <V3d_DirectionalLight.hxx>
#include <V3d_AmbientLight.hxx>

static Handle(Graphic3d_GraphicDriver) Viewer_aGraphicDriver;

Viewer::Viewer(QWidget *parent):QWidget(parent)
{
    //Linux Window
    initXW();

    //occ
    if(myGraphicDriver.IsNull())
    {
        if(Viewer_aGraphicDriver.IsNull())
        {
            Handle(Aspect_DisplayConnection) aDisplayConnection;
            aDisplayConnection = new Aspect_DisplayConnection(qgetenv("DISPLAY").constData());
            Viewer_aGraphicDriver = new OpenGl_GraphicDriver(aDisplayConnection);
        }
        myGraphicDriver = Handle(OpenGl_GraphicDriver)::DownCast(Viewer_aGraphicDriver);
    }

    Handle(V3d_Viewer) aViewer = new V3d_Viewer(myGraphicDriver,TCollection_ExtendedString("Visu3D").ToExtString(),"",1000,V3d_XposYnegZpos,Quantity_NOC_GRAY30,V3d_ZBUFFER,V3d_GOURAUD,V3d_WAIT,true,true,V3d_TEX_NONE);
    aViewer->SetDefaultBackgroundColor(Quantity_NOC_BLACK);
    aViewer->SetDefaultLights();
    aViewer->SetLightOn();

    myView = aViewer->CreateView();

    myIC = new AIS_InteractiveContext(aViewer);
    myIC->SetDeviationCoefficient(1.e-3);

    Window aWindowHandle = (Window)winId();
    Handle(Aspect_DisplayConnection) aDispConnection = myIC->CurrentViewer()->Driver()->GetDisplayConnection();
    Handle(Xw_Window) hWnd = new Xw_Window(aDispConnection,aWindowHandle);

    myView->SetWindow(hWnd);
    if(!hWnd->IsMapped())
    {
        hWnd->Map();
    }

    myView->MustBeResized();
    myView->SetSurfaceDetail(V3d_TEX_NONE);
    myView->SetTransparency(Standard_True);
    myView->SetSize(10000.0);
    myView->SetZSize(10000.0);
    myView->SetViewMappingDefault();

    setMouseTracking(true);
    setMinimumSize(400,200);

    myView->ZBufferTriedronSetup();

    myView->SetBackgroundColor(Quantity_NOC_BLACK);
    myView->TriedronDisplay(Aspect_TOTP_LEFT_LOWER,Quantity_NOC_GOLD,0.1,V3d_ZBUFFER);

    setBackgroundRole(QPalette::NoRole);
    setFocusPolicy(Qt::StrongFocus);
    setAttribute(Qt::WA_PaintOnScreen);
    setAttribute(Qt::WA_NoSystemBackground);

    aViewer->SetLightOn(new V3d_DirectionalLight(aViewer,V3d_XnegYnegZneg,Quantity_NOC_WHITE, Standard_True));
    aViewer->SetLightOn(new V3d_AmbientLight(aViewer,Quantity_NOC_WHITE));


}

Viewer::~Viewer(){}

void Viewer::initXW()
{
    XSynchronize(x11Info().display(),true);
    XVisualInfo *pVisualInfo;
    if(x11Info().display())
    {
        Visual *v = DefaultVisual(x11Info().display(),DefaultScreen(x11Info().display()));
        int visualID = XVisualIDFromVisual(v);
        int visualAttr[] = {GLX_RGBA,GLX_DEPTH_SIZE,1,GLX_RED_SIZE,1,GLX_GREEN_SIZE,1,GLX_BLUE_SIZE,1,GLX_DOUBLEBUFFER,None};
        pVisualInfo = ::glXChooseVisual(x11Info().display(),DefaultScreen(x11Info().display()),visualAttr);

        if(isVisible())
            hide();

        XSetWindowAttributes a;
        Window p = RootWindow(x11Info().display(),DefaultScreen(x11Info().display()));
        a.colormap = XCreateColormap(x11Info().display(),RootWindow(x11Info().display(),pVisualInfo->screen),pVisualInfo->visual,AllocNone);
        QColor color = palette().color(backgroundRole());
        QColormap colmap = QColormap::instance();
        a.background_pixel = colmap.pixel(color);
        a.border_pixel = colmap.pixel(Qt::black);

        if(parentWidget())
            p = parentWidget()->winId();
        Window w = XCreateWindow(x11Info().display(),p,x(),y(),width(),height(),0,pVisualInfo->depth,InputOutput,pVisualInfo->visual,CWBackPixel|CWBorderPixel|CWColormap,&a);
        Window *cmw;
        Window *cmwret;
        int count;
        if(XGetWMColormapWindows(x11Info().display(),topLevelWidget()->winId(),&cmwret,&count))
        {
            cmw = new Window[count+1];
            memcpy((char *)cmw,(char *)cmwret,sizeof(Window)*count);
            XFree((char *)cmwret);
            int i;
            for (i=0;i<count;i++)
            {
                if(cmw[i]==winId())
                {
                    cmw[i]=w;
                    break;
                }
            }
            if(i>=count)
                cmw[count++] = w;
        }
        else
        {
            count = 1;
            cmw = new Window[count];
            cmw[0]=w;
        }
        create(w);

        XSetWMColormapWindows(x11Info().display(),topLevelWidget()->winId(),cmw,count);
        delete [] cmw;
        if(isVisible())
            show();
        if(pVisualInfo)
            XFree((char *)pVisualInfo);
        XFlush(x11Info().display());
    }

}

void Viewer::paintEvent(QPaintEvent *e)
{
    myView->Redraw();
}

void Viewer::resizeEvent(QResizeEvent *e)
{
    if(!myView.IsNull())
    {
        myView->MustBeResized();
    }
}
